@@ -550,6 +550,7 @@ enum fb_write_logical_srcs {
    FB_WRITE_LOGICAL_SRC_SRC_STENCIL, /* gl_FragStencilRefARB */
    FB_WRITE_LOGICAL_SRC_OMASK,       /* Sample Mask (gl_SampleMask) */
    FB_WRITE_LOGICAL_SRC_COMPONENTS,  /* REQUIRED */
+   FB_WRITE_LOGICAL_SRC_NULL_RT,     /* Null RT write */
    FB_WRITE_LOGICAL_NUM_SRCS
 };
 
@@ -954,7 +955,8 @@ tgl_swsb_src_dep(struct tgl_swsb swsb)
  * SWSB annotation.
  */
 static inline uint32_t
-tgl_swsb_encode(const struct intel_device_info *devinfo, struct tgl_swsb swsb)
+tgl_swsb_encode(const struct intel_device_info *devinfo,
+                struct tgl_swsb swsb, enum opcode opcode)
 {
    if (!swsb.mode) {
       const unsigned pipe = devinfo->verx10 < 125 ? 0 :
@@ -967,18 +969,27 @@ tgl_swsb_encode(const struct intel_device_info *devinfo, struct tgl_swsb swsb)
 
    } else if (swsb.regdist) {
       if (devinfo->ver >= 20) {
-         if ((swsb.mode & TGL_SBID_SET)) {
+         unsigned mode = 0;
+         if (opcode == BRW_OPCODE_DPAS) {
+            mode = (swsb.mode & TGL_SBID_SET) ? 0b01 :
+                   (swsb.mode & TGL_SBID_SRC) ? 0b10 :
+                 /* swsb.mode & TGL_SBID_DST */ 0b11;
+         } else if (swsb.mode & TGL_SBID_SET) {
+            assert(opcode == BRW_OPCODE_SEND || opcode == BRW_OPCODE_SENDC);
             assert(swsb.pipe == TGL_PIPE_ALL ||
-                   swsb.pipe == TGL_PIPE_INT || swsb.pipe == TGL_PIPE_FLOAT);
-            return (swsb.pipe == TGL_PIPE_INT ? 0x300 :
-                    swsb.pipe == TGL_PIPE_FLOAT ? 0x200 : 0x100) |
-                   swsb.regdist << 5 | swsb.sbid;
+                   swsb.pipe == TGL_PIPE_INT ||
+                   swsb.pipe == TGL_PIPE_FLOAT);
+
+            mode = swsb.pipe == TGL_PIPE_INT   ? 0b11 :
+                   swsb.pipe == TGL_PIPE_FLOAT ? 0b10 :
+                /* swsb.pipe == TGL_PIPE_ALL  */ 0b01;
          } else {
             assert(!(swsb.mode & ~(TGL_SBID_DST | TGL_SBID_SRC)));
-            return (swsb.pipe == TGL_PIPE_ALL ? 0x300 :
-                    swsb.mode == TGL_SBID_SRC ? 0x200 : 0x100) |
-                   swsb.regdist << 5 | swsb.sbid;
+            mode = swsb.pipe == TGL_PIPE_ALL  ? 0b11 :
+                   swsb.mode == TGL_SBID_SRC  ? 0b10 :
+                /* swsb.mode == TGL_SBID_DST */ 0b01;
          }
+         return mode << 8 | swsb.regdist << 5 | swsb.sbid;
       } else {
          assert(!(swsb.sbid & ~0xfu));
          return 0x80 | swsb.regdist << 4 | swsb.sbid;
@@ -1002,11 +1013,12 @@ tgl_swsb_encode(const struct intel_device_info *devinfo, struct tgl_swsb swsb)
  */
 static inline struct tgl_swsb
 tgl_swsb_decode(const struct intel_device_info *devinfo,
-                const bool is_unordered, const uint32_t x)
+                const bool is_unordered, const uint32_t x, enum opcode opcode)
 {
    if (devinfo->ver >= 20) {
       if (x & 0x300) {
-         if (is_unordered) {
+         /* Mode isn't SingleInfo, there's a tuple */
+         if (opcode == BRW_OPCODE_SEND || opcode == BRW_OPCODE_SENDC) {
             const struct tgl_swsb swsb = {
                (x & 0xe0u) >> 5,
                ((x & 0x300) == 0x300 ? TGL_PIPE_INT :
@@ -1016,6 +1028,16 @@ tgl_swsb_decode(const struct intel_device_info *devinfo,
                TGL_SBID_SET
             };
             return swsb;
+         } else if (opcode == BRW_OPCODE_DPAS) {
+            const struct tgl_swsb swsb = {
+               .regdist = (x & 0xe0u) >> 5,
+               .pipe = TGL_PIPE_NONE,
+               .sbid = x & 0x1fu,
+               .mode = (x & 0x300) == 0x300 ? TGL_SBID_DST :
+                       (x & 0x300) == 0x200 ? TGL_SBID_SRC :
+                                              TGL_SBID_SET,
+            };
+            return swsb;
          } else {
             const struct tgl_swsb swsb = {
                (x & 0xe0u) >> 5,
